@@ -1,5 +1,6 @@
 from __future__ import print_function
 
+import os
 import base64
 import json
 import time
@@ -7,119 +8,33 @@ from time import sleep, time
 
 import requests
 
-from intellihub.assertions import validate_parameters, is_url_valid, allowed_file_extension
+from IPython.display import display
+from PIL import Image
+
+from intellihub.assertions import validate_parameters, is_url_valid, allowed_file_extension, hyper_parameter_check
 from intellihub.dataset_types import Dataset
 
-def register(email, mobile_number, first_name, last_name, password):
-        payload = {
-            "email" : email,
-            "mobile" : mobile_number,
-            "firstName": first_name,
-            "lastName": last_name,
-            "password": password,
-            "accessLevel": "USER",
-            "roles": [
-                {
-                    "id": 1,
-                    
-                }
-            ],
-            "attributes": {},
-            "clientIds": [
-                "intellihub"
-            ]
-        }
-        url = "https://dev-kong.intellihub.ai/register/otp"
-        body = json.dumps(payload)
-        headers = {
-        'Content-Type': 'application/json'
-        }
-        response = requests.post(url=url, data=body, headers=headers)
-        return response
 
-def registration_confirm(token_value):
-    url = "https://dev-kong.intellihub.ai/registrationConfirm?token=" + str(token_value)
-    response = requests.get(url=url)
-    return response
-
-def forgot_password(email):
-    url = "https://dev-kong.intellihub.ai/user/forgotPassword/otp?email=" + str(email)
-    response = requests.post(url=url)
-    return response
-
-def reset_password(email, token, new_password):
-    url = f"https://dev-kong.intellihub.ai/user/resetPassword/otp?email={email}&token={token}"
-
-    payload = json.dumps({
-    "newPassword": new_password
-    })
-    headers = {
-    'Content-Type': 'application/json'
-    }
-    response = requests.post(url=url, data=payload, headers=headers)
-    return response
-
-def initialize(authentication_token):
-    url = "https://dev-kong.intellihub.ai/user/initialize"
-
-    headers = {'AuthenticationToken': authentication_token}
-    response = requests.post(url=url, headers=headers)
-    return response
-
-def login(user_name, password):
-    url = "https://dev-kong.intellihub.ai/oauth/login"
-
-    payload= f'grant_type=password&client_id=intellihub&username={user_name}&password={password}'
-    headers = {
-    'Authorization': 'Basic aW50ZWxsaWh1YjppbnRlbGxpaHVi',
-    'Content-Type': 'application/x-www-form-urlencoded'
-    }
-
-    response = requests.post(url=url, data=payload, headers=headers)
-    json_response = json.loads(response.text)
-    access_token = json_response["access_token"]
-    initialize(access_token)
-    return response
-
-def create_app(app_name, description, access_token):
-    url = "https://dev-kong.intellihub.ai/app/create/"
-    payload = json.dumps({
-        "name": app_name,
-        "description": description
-    })
-    headers = {'AuthenticationToken': access_token, 'Content-Type': 'application/json'}
-    response = requests.post(url=url, data=payload, headers=headers)
-    return response
-
-def app_list(access_token):
-    url = "https://dev-kong.intellihub.ai/app/list"
-    headers = {'AuthenticationToken': access_token}
-    response = requests.get(url=url, headers=headers)
-    return response
-
-class IntellihubClient:
+class IntellihubAiClient:
     """
 
         Attributes:
-            api_key (str): API Key Generated for an app in Intellihub.
+            api_key (str): API Key Generated for an app in IntellihubAi.
     """
 
-    def __init__(self, api_key=None, base_url="http://14.1.1.75:8000"):
+    def __init__(self, api_key=None, base_url="https://prod-kong.intellihub"):
         """
-            The constructor for Intellihub Client.
+            The constructor for IntellihubAi Client.
 
             Parameters:
-                api_key (str): API Key Generated for an app in Intellihub.
+                api_key (str): API Key Generated for an app in IntellihubAi.
 
             Returns:
-                IntellihubClient: Client object for Intellihub.
+                IntellihubAiClient: Client object for IntellihubAi.
         """
         self.api_key = api_key
         self.base_url = base_url
 
-    #Note: Signup functions
-    
-
     # Note: NLP functions
 
     def sentiment_analysis(self, text, sources=['nltk_vader'], **kwargs):
@@ -261,7 +176,7 @@ class IntellihubClient:
             task_status = "PENDING"
 
             start_time = time()
-            while task_status != "SUCCESS":
+            while task_status == "PENDING":
                 task_status_response = requests.get(task_url + task_creation_response['job_id'], headers=headers).json()
                 task_status = task_status_response["task_status"]
                 # wait for some time before checking status of the job again
@@ -296,7 +211,7 @@ class IntellihubClient:
         assert image_url is not None or image_path is not None, "Please choose either image_url or image_path"
         if image_url is not None:
             assert is_url_valid(image_url), "Enter a valid URL"
-        supported_object_detectors = ['tensorflow', 'azure', 'ibm']
+        supported_object_detectors = ['tensorflow', 'azure']
         assert len(supported_object_detectors) >= 1, f"Please select object_detectors from {supported_object_detectors}"
 
         assert len([classifier for classifier in object_detectors if classifier in supported_object_detectors]) > 0, f"Please select object_detectors from {supported_object_detectors}"
@@ -339,7 +254,7 @@ class IntellihubClient:
 
         return response
 
-    def image_classification(self, image_url=None, image_path=None, top_n=3, image_classifiers=[],
+    def image_classification(self, image_url=None, image_path=None, top_n=3, image_classifiers=['tensorflow'],
                              output_types=["json"], reformat=True, wait_time=10):
         """
         This function is for image classification
@@ -440,10 +355,10 @@ class IntellihubClient:
             raise Exception('Error while checking the query list. Got ' + str(response.status_code))
         return response
 
-    def train(self, service, algorithm, dataset, label, features, model_name=None, lib="weka", train_percentage=80, save_model=True, folds=5, cross_validation=False, params=None, dataset_source=None):
+    def train(self, task, algorithm, dataset, label, features, model_name=None, lib="weka", train_percentage=80,  folds=5, cross_validation=False, params=None, dataset_source=None, evaluation_plots=False):
 
         """
-        :param service: Training task to perform. Valid parameter values are classification, regression.
+        :param task: Training task to perform. Valid parameter values are classification, regression.
         :param algorithm: Algorithm used for training the model.
         :param dataset: dataset file location in INTELLIHUB storage.
         :param label: Target variable.
@@ -451,7 +366,6 @@ class IntellihubClient:
         :param model_name: Model will be saved with the name specified in this parameter.
         :param lib: Library for training the model. Currently we are supporting scikit, h2o and weka.
         :param train_percentage: Percentage of data used for training the model. Rest of the data will be used to test the model.
-        :param save_model: If True, the model will be saved in the INTELLIHUB Storage.
         :param dataset_source: To specify data source,
                 None: Dataset file from INTELLIHUB storage will be used
                 database: Query from connected database will be used
@@ -461,11 +375,15 @@ class IntellihubClient:
         
         """
 
+        task, library, algorithm, features, label, train_percentage = validate_parameters(
+            task, lib, algorithm, features, label, train_percentage)
 
-        service, library, algorithm, features, label, train_percentage, save_model = validate_parameters(
-            service, lib, algorithm, features, label, train_percentage)
+        # if additional parameters passed, check whether those are valid or not
+        if params is not None:
+            hyper_parameter_flag = hyper_parameter_check(library, task, algorithm, params)
+            assert hyper_parameter_flag, "Please check the params, training failed due to incorrect values"
 
-        url = self.base_url + '/machine/' + service + '/train/'
+        url = self.base_url + '/machine/' + task + '/train/'
         headers = {"ApiKey": self.api_key, "Content-type": "application/json"}
         if params is None:
             params = {}
@@ -484,10 +402,10 @@ class IntellihubClient:
                     "label": label,
                     "trainPercentage": train_percentage,
                     "features": features,
-                    "saveModel": save_model,
                     "params": params,
                     "folds" : folds,
-                    "crossValidation" : cross_validation
+                    "crossValidation" : cross_validation,
+                    "evalPlots": evaluation_plots
                 }
             }
         else:
@@ -501,10 +419,10 @@ class IntellihubClient:
                     "label": label,
                     "trainPercentage": train_percentage,
                     "features": features,
-                    "saveModel": save_model,
                     "params": params,
                     "folds" : folds,
-                    "crossValidation" : cross_validation
+                    "crossValidation" : cross_validation,
+                    "evalPlots": evaluation_plots
                 }
             }
         body = json.dumps(body)
@@ -512,10 +430,10 @@ class IntellihubClient:
         response = response.json()
         return response
 
-    def feedback(self, service, algorithm, train_data, feedback_data, job_id, model_url, label, features, lib='weka',
-                 model_name=None, split_perc=80,save_model=True, folds=5, cross_validation=False, params=None):
+    def feedback(self, task, algorithm, train_data, feedback_data, job_id, model_url, label, features, lib='weka',
+                 model_name=None, split_perc=80, folds=5, cross_validation=False, params=None, evaluation_plots=False):
         """
-        :param service: Training task to perform. Valid parameter values are classification, regression.
+        :param task: Training task to perform. Valid parameter values are classification, regression.
         :param algorithm: Algorithm used for training the model.
         :param train_data: dataset file location in INTELLIHUB storage.
         :param feedback_data: dataset file location in INTELLIHUB storage.
@@ -526,16 +444,15 @@ class IntellihubClient:
         :param lib: Library for training the model. Currently we are supporting scikit, h2o and weka.
         :param model_name: Model will be saved with the name specified in this parameter.
         :param split_perc: Percentage of data to use for training the model. Rest of the data will be used to test the model.
-        :param save_model: If True, the model will be saved in INTELLIHUB Storage.
         :param params: additional parameters.
         :param folds: number of folds for cross validation
         :param cross_validation: Evaluates model using crossvalidation if set to True.
 
         :rtype: A json object containing the file path in storage.
         """
-        service, library, algorithm, features, label, train_percentage, save_model = validate_parameters(
-            service, lib, algorithm, features, label)
-        url = self.base_url + '/machine/' + service + '/feedback'
+        task, library, algorithm, features, label, train_percentage = validate_parameters(
+            task, lib, algorithm, features, label)
+        url = self.base_url + '/machine/' + task + '/feedback'
 
         headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'}
 
@@ -547,7 +464,7 @@ class IntellihubClient:
 
         body = {
             'library': lib,
-            'service': service,
+            'service': task,
             'task': 'FEEDBACK',
             'config': {
                 'jobId': job_id,
@@ -560,19 +477,19 @@ class IntellihubClient:
                 'trainPercentage': split_perc,
                 'features': features,
                 'params': params,
-                'saveModel': save_model,
                 "folds" : folds,
-                "crossValidation" : cross_validation
+                "crossValidation" : cross_validation,
+                "evalPlots": evaluation_plots
             }
         }
         body = json.dumps(body)
         response = requests.post(url=url, data=body, headers=headers)
         return response.json()
 
-    def predict(self, service, dataset, model_url, features, lib='weka', params=None, dataset_source=None):
+    def predict(self, task, dataset, model_url, features, lib='weka', params=None, dataset_source=None):
         """
         :param lib: Library for training the model. Currently we are supporting INTELLIHUB and weka libraries.
-        :param service: Valid parameter values are classification, regression.
+        :param task: Valid parameter values are classification, regression.
         :param dataset: dataset file location in INTELLIHUB storage.
         :param model_url: trained model location in INTELLIHUB storage.
         :param features: list of features used for training
@@ -587,13 +504,13 @@ class IntellihubClient:
                 database: Query from connected database will be used
 
         """
-        service, library, algorithm, features, label, train_percentage, save_model = validate_parameters(
-            service, lib, algorithm=None,
+        task, library, algorithm, features, label, train_percentage= validate_parameters(
+            task, lib, algorithm=None,
             features=features,
             label=None,
             cluster=True,
             predict=True)
-        url = self.base_url + '/machine/' + service + '/predict'
+        url = self.base_url + '/machine/' + task + '/predict'
         headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'}
         if params is None:
             params = {}
@@ -624,8 +541,8 @@ class IntellihubClient:
         response = requests.post(url=url, data=body, headers=headers).json()
         return response
 
-    def cluster(self, service, algorithm, dataset, features, lib='weka', number_of_clusters=2, model_name=None,
-                save_model=True, params=None, dataset_source=None):
+    def cluster(self, service, algorithm, dataset, features, lib='weka', number_of_clusters=2, cluster_type="Centroid", model_name=None,
+                 params=None, dataset_source=None):
         """
         :param lib: Library for clustering the model. Currently we are supporting INTELLIHUB, weka, H2O, scikit-learn
                     libraries. Valid values for this parameter: INTELLIHUB, weka, h2o, scikit
@@ -635,7 +552,6 @@ class IntellihubClient:
         :param dataset: dataset file location in INTELLIHUB storage.
         :param features: column name list which is used to train classification model.
         :param number_of_clusters: the dataset will be clustered into number of clusters.
-        :param save_model: If true model will saved
         :param dataset_source : metabase address for dataset
         :param params:
         :return:
@@ -649,9 +565,14 @@ class IntellihubClient:
                 None: Dataset file will from INTELLIHUB storage will be used
                 database: Query from connected database will be used
         """
-        service, library, algorithm, features, label, train_percentage, save_model = validate_parameters(
-            service, lib, algorithm, features,
-            save_model, cluster=True)
+        service, library, algorithm, features, label, train_percentage = validate_parameters(
+            service, lib, algorithm, features, "None", cluster=True)
+
+        # if additional parameters passed, check whether those are valid or not
+        if params is not None:
+            hyper_parameter_flag = hyper_parameter_check(library, service, algorithm, params)
+            assert hyper_parameter_flag, "Please check the params, training failed due to incorrect values"
+
         url = self.base_url + '/machine/cluster/'
         headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'}
         if params is None:
@@ -671,8 +592,8 @@ class IntellihubClient:
                     'numOfClusters': int(number_of_clusters),
                     'epsilon': 0.1,
                     'features': features,
-                    'saveModel': save_model,
-                    'params': params
+                    'params': params,
+                    'clusterType':cluster_type
                 }
             }
         else:
@@ -687,7 +608,136 @@ class IntellihubClient:
                     'numOfClusters': int(number_of_clusters),
                     'epsilon': 0.1,
                     'features': features,
-                    'saveModel': save_model,
+                    'params': params,
+                    'clusterType': cluster_type
+                }
+            }
+        body = json.dumps(body)
+        response = requests.post(url=url, data=body, headers=headers)
+        response = response.json()
+        return response
+
+    def clusterRecommendation(self, file_url, features, algorithm):
+
+        allowed_algorithms = ["SimpleKMeans", "FarthestFirst", "Canopy", "Cobweb", "HierarchicalClusterer", "KMeansClustering", "MiniBatchKMeans", "Birch", "AgglomerativeClustering", "SpectralClustering","DBScan", "Optics", "GaussianMixtures"]
+
+        assert algorithm in allowed_algorithms, "Please select algorithm from {}".format(allowed_algorithms)
+
+        url = self.base_url + '/intellihub-ai-ml-recommendation/cluster/cluster-recommendation'
+        data = {"fileUrl": file_url, "features": features, "algorithm": algorithm}
+        data = json.dumps(data)
+
+
+        if algorithm in ["SimpleKMeans", "FarthestFirst", "Canopy"]:
+            algorithm = "KMeansClustering"
+        if algorithm in ["Cobweb", "HierarchicalClusterer"]:
+            algorithm = "AgglomerativeClustering"
+
+        headers = {
+            'ApiKey': self.api_key,
+            'Content-Type': 'application/json'
+        }
+
+        response = requests.request("POST", url, headers=headers, data=data)
+        response = json.loads(response.text)
+        if response['success'] ==  True:
+
+            if algorithm not in ['DBScan','Optics']:
+
+                imgdata = base64.b64decode(response['response'])
+                filename = 'recommendation.jpg'
+                with open(filename, 'wb') as f:
+                    f.write(imgdata)
+                display(Image.open('recommendation.jpg'))
+                os.remove('recommendation.jpg')
+
+            else:
+
+                imgdata_epsilon = base64.b64decode(response['response']['epsilon'])
+                filename = 'recommendation_epsilon.jpg'
+                with open(filename, 'wb') as f:
+                    f.write(imgdata_epsilon)
+                display(Image.open('recommendation_epsilon.jpg'))
+
+                imgdata_min_samples = base64.b64decode(response['response']['min_samples'])
+                filename = 'recommendation_min_samples.jpg'
+                with open(filename, 'wb') as f:
+                    f.write(imgdata_min_samples)
+                display(Image.open('recommendation_min_samples.jpg'))
+                os.remove('recommendation_epsilon.jpg')
+                os.remove('recommendation_min_samples.jpg')
+
+        else:
+            return response.text
+
+
+
+
+    def clusterPredict(self, service, algorithm, dataset, features, model_url, lib='weka', number_of_clusters=2, model_name=None,
+                 params=None, dataset_source=None):
+        """
+        :param lib: Library for clustering the model. Currently we are supporting INTELLIHUB, weka, H2O, scikit-learn
+                    libraries. Valid values for this parameter: INTELLIHUB, weka, h2o, scikit
+        :param service: Valid parameter values are CLUSTER.
+        :param model_name: Model name and with this name model will be saved.
+        :param algorithm: algorithm by which model will be trained.
+        :param dataset: dataset file location in INTELLIHUB storage.
+        :param features: column name list which is used to train classification model.
+        :param model_url: model file location in INTELLIHUB storage.
+        :param number_of_clusters: the dataset will be clustered into number of clusters.
+        :param dataset_source : metabase address for dataset
+        :param params:
+        :return:
+            obj: A json obj containing model info.
+
+        Args:
+            dataset_source:
+            dataset_source:
+            features: Feature list used while model training
+            dataset_source: To specify data source,
+                None: Dataset file will from INTELLIHUB storage will be used
+                database: Query from connected database will be used
+        """
+        service, library, algorithm, features, label, train_percentage = validate_parameters(
+            service, lib, algorithm, features, "None",
+             cluster=True)
+
+        url = self.base_url + '/machine/cluster/predict'
+        headers = {'ApiKey': self.api_key, 'Content-type': 'application/json'}
+        if params is None:
+            params = {}
+        if model_name is None:
+            model_name = algorithm
+        if dataset_source == "database":
+            body = {
+                'library': lib,
+                'task': 'CLUSTER',
+                'service': service,
+                "jobType": "DATABASE",
+                "queryId": dataset,
+                'config': {
+                    'name': model_name,
+                    'algorithm': algorithm,
+                    'modelUrl': model_url,
+                    'numOfClusters': int(number_of_clusters),
+                    'epsilon': 0.1,
+                    'features': features,
+                    'params': params
+                }
+            }
+        else:
+            body = {
+                'library': lib,
+                'task': 'CLUSTER',
+                'service': service,
+                'config': {
+                    'name': model_name,
+                    'algorithm': algorithm,
+                    'modelUrl': model_url,
+                    'datasetUrl': dataset,
+                    'numOfClusters': int(number_of_clusters),
+                    'epsilon': 0.1,
+                    'features': features,
                     'params': params
                 }
             }
@@ -696,9 +746,10 @@ class IntellihubClient:
         response = response.json()
         return response
 
-    def job_status(self, job_id):
+    def job_status(self, job_id, wait_time = 20):
         """
         :param job_id: jobId from the train api response.
+        :param wait_time: wait time for server to return response
         :return:
             obj: A json obj containing the status details.
         """
@@ -709,9 +760,15 @@ class IntellihubClient:
         response = requests.get(url=url, headers=headers)
         if response.status_code == 200:
             response = response.json()
+            start_time = time()
             while response[STATE] == 'RUN':
                 sleep(JOB_STATUS_CHECK_INTERVAL)
                 response = requests.get(url=url, headers=headers).json()
+                if time() - start_time > wait_time:
+                    job_id = response.json()["jobId"]
+                    print("It's taking too long than expected!!",
+                          f"Use job_status('{job_id}') to check status of your request")
+                    break
             if response[STATE] == 'FAIL':
                 raise Exception('Prediction job failed!')
         else:
@@ -769,7 +826,7 @@ class IntellihubClient:
             features: list of features requested by client
             face_detectors: supported face detectors are mtcnn, dlib, opencv, azure
             wait_time: wait time for server to return response
-1`
+
         Returns:
         face analytics response dependent on the features requested by client
         """
